@@ -1,6 +1,9 @@
+# frozen_string_literal: true
+
 # utility classes
 require 'aws-sdk-s3/bucket_region_cache'
 require 'aws-sdk-s3/encryption'
+require 'aws-sdk-s3/encryption_v2'
 require 'aws-sdk-s3/file_part'
 require 'aws-sdk-s3/file_uploader'
 require 'aws-sdk-s3/file_downloader'
@@ -26,6 +29,6 @@ require 'aws-sdk-s3/customizations/types/list_object_versions_output'
   Aws::S3::ObjectVersion::Collection,
 ].each do |klass|
   klass.send(:alias_method, :delete, :batch_delete!)
-  klass.send(:extend, Aws::Deprecations)
+  klass.extend Aws::Deprecations
   klass.send(:deprecated, :delete, use: :batch_delete!)
 end
