@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'pathname'
 require 'thread'
 require 'set'
@@ -21,7 +23,7 @@ module Aws
 
       def download(destination, options = {})
         @path = destination
-        @mode = options[:mode] || "auto"
+        @mode = options[:mode] || 'auto'
         @thread_count = options[:thread_count] || THREAD_COUNT
         @chunk_size = options[:chunk_size]
         @params = {
@@ -31,19 +33,19 @@ module Aws
         @params[:version_id] = options[:version_id] if options[:version_id]
 
         case @mode
-        when "auto" then multipart_download
-        when "single_request" then single_request
-        when "get_range"
+        when 'auto' then multipart_download
+        when 'single_request' then single_request
+        when 'get_range'
           if @chunk_size
             resp = @client.head_object(@params)
             multithreaded_get_by_ranges(construct_chunks(resp.content_length))
           else
-            msg = "In :get_range mode, :chunk_size must be provided"
+            msg = 'In :get_range mode, :chunk_size must be provided'
             raise ArgumentError, msg
           end
         else
           msg = "Invalid mode #{@mode} provided, "\
-            "mode should be :single_request, :get_range or :auto"
+                'mode should be :single_request, :get_range or :auto'
           raise ArgumentError, msg
         end
       end
@@ -92,7 +94,12 @@ module Aws
         if @chunk_size && @chunk_size > file_size
           raise ArgumentError, ":chunk_size shouldn't exceed total file size."
         else
-          @chunk_size || [(file_size.to_f / MAX_PARTS).ceil, MIN_CHUNK_SIZE].max.to_i
+          chunk_size = @chunk_size || [
+            (file_size.to_f / MAX_PARTS).ceil,
+            MIN_CHUNK_SIZE
+          ].max.to_i
+          chunk_size -= 1 if file_size % chunk_size == 1
+          chunk_size
         end
       end
 
@@ -125,9 +132,9 @@ module Aws
       end
 
       def write(resp)
-        range, _ = resp.content_range.split(" ").last.split("/")
-        head, _ = range.split("-").map {|s| s.to_i}
-        IO.write(@path, resp.body.read, head)
+        range, _ = resp.content_range.split(' ').last.split('/')
+        head, _ = range.split('-').map {|s| s.to_i}
+        File.write(@path, resp.body.read, head)
       end
 
       def single_request
